(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/common')) :
    typeof define === 'function' && define.amd ? define('mutable-div', ['exports', '@angular/core', '@angular/common'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global['mutable-div'] = {}, global.ng.core, global.ng.common));
}(this, (function (exports, core, common) { 'use strict';

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    var DraggableElementDirective = /** @class */ (function () {
        /**
         * @param {?} el
         */
        function DraggableElementDirective(el) {
            this.el = el;
            this.pointerDown = false;
            this.rotate = 0;
            this.coordinates = new core.EventEmitter();
            this.stopped = new core.EventEmitter();
            el.nativeElement.style.position = 'absolute';
        }
        /**
         * @param {?} event
         * @return {?}
         */
        DraggableElementDirective.prototype.onPointerDown = function (event) {
            event.preventDefault();
            this.pointerDown = true;
            this.setOriginalPosition(event);
        };
        /**
         * @return {?}
         */
        DraggableElementDirective.prototype.onPointerUp = function () {
            this.stopped.emit();
        };
        /**
         * @param {?} event
         * @return {?}
         */
        DraggableElementDirective.prototype.onPointerMove = function (event) {
            if (this.pointerDown) {
                /** @type {?} */
                var element = this.el.nativeElement;
                /** @type {?} */
                var newPosition = this.setNewPosition(event);
                this.setOriginalPosition(event);
                element.style.left = (element.offsetLeft + newPosition.x) + 'px';
                element.style.top = (element.offsetTop + newPosition.y) + 'px';
                this.coordinates.emit({ x: newPosition.x, y: newPosition.y });
            }
        };
        /**
         * @return {?}
         */
        DraggableElementDirective.prototype.onWindowPointerUp = function () {
            this.pointerDown = false;
        };
        /**
         * @param {?} event
         * @return {?}
         */
        DraggableElementDirective.prototype.onPanStart = function (event) {
            this.onPointerDown(event.srcEvent);
        };
        /**
         * @return {?}
         */
        DraggableElementDirective.prototype.onPanEnd = function () {
            this.onPointerUp();
        };
        /**
         * @param {?} event
         * @return {?}
         */
        DraggableElementDirective.prototype.onPanMove = function (event) {
            this.onPointerMove(event.srcEvent);
        };
        /**
         * @return {?}
         */
        DraggableElementDirective.prototype.onWindowPanEnd = function () {
            this.onWindowPointerUp();
        };
        /**
         * @private
         * @param {?} event
         * @return {?}
         */
        DraggableElementDirective.prototype.setOriginalPosition = function (event) {
            switch (this.rotate) {
                case 90:
                    this.originalPosition = {
                        x: event.clientY,
                        y: screen.width - event.clientX
                    };
                    break;
                case 180:
                    this.originalPosition = {
                        x: screen.width - event.clientX,
                        y: screen.height - event.clientY
                    };
                    break;
                case 270:
                    this.originalPosition = {
                        x: screen.height - event.clientY,
                        y: event.clientX
                    };
                    break;
                default:
                    this.originalPosition = {
                        x: event.clientX,
                        y: event.clientY
                    };
            }
        };
        /**
         * @private
         * @param {?} event
         * @return {?}
         */
        DraggableElementDirective.prototype.setNewPosition = function (event) {
            /** @type {?} */
            var newPosition = {
                x: 0,
                y: 0
            };
            switch (this.rotate) {
                case 90:
                    newPosition = {
                        x: event.clientY - this.originalPosition.x,
                        y: (screen.width - event.clientX) - this.originalPosition.y
                    };
                    break;
                case 180:
                    newPosition = {
                        x: (screen.width - event.clientX) - this.originalPosition.x,
                        y: (screen.height - event.clientY) - this.originalPosition.y
                    };
                    break;
                case 270:
                    newPosition = {
                        x: (screen.height - event.clientY) - this.originalPosition.x,
                        y: event.clientX - this.originalPosition.y
                    };
                    break;
                default:
                    newPosition = {
                        x: event.clientX - this.originalPosition.x,
                        y: event.clientY - this.originalPosition.y
                    };
            }
            return newPosition;
        };
        return DraggableElementDirective;
    }());
    DraggableElementDirective.decorators = [
        { type: core.Directive, args: [{
                    selector: '[draggable]'
                },] }
    ];
    /** @nocollapse */
    DraggableElementDirective.ctorParameters = function () { return [
        { type: core.ElementRef }
    ]; };
    DraggableElementDirective.propDecorators = {
        rotate: [{ type: core.Input }],
        coordinates: [{ type: core.Output }],
        stopped: [{ type: core.Output }],
        onPointerDown: [{ type: core.HostListener, args: ['pointerdown', ['$event'],] }],
        onPointerUp: [{ type: core.HostListener, args: ['pointerup',] }],
        onPointerMove: [{ type: core.HostListener, args: ['window:pointermove', ['$event'],] }],
        onWindowPointerUp: [{ type: core.HostListener, args: ['window:pointerup',] }],
        onPanStart: [{ type: core.HostListener, args: ['panstart', ['$event'],] }],
        onPanEnd: [{ type: core.HostListener, args: ['panend',] }],
        onPanMove: [{ type: core.HostListener, args: ['window:panmove', ['$event'],] }],
        onWindowPanEnd: [{ type: core.HostListener, args: ['window:panend',] }]
    };
    if (false) {
        /** @type {?} */
        DraggableElementDirective.prototype.originalPosition;
        /**
         * @type {?}
         * @private
         */
        DraggableElementDirective.prototype.pointerDown;
        /** @type {?} */
        DraggableElementDirective.prototype.rotate;
        /** @type {?} */
        DraggableElementDirective.prototype.coordinates;
        /** @type {?} */
        DraggableElementDirective.prototype.stopped;
        /**
         * @type {?}
         * @private
         */
        DraggableElementDirective.prototype.el;
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    var ResizeHandlersComponent = /** @class */ (function () {
        function ResizeHandlersComponent() {
            this.selected = false;
            this.minimumSize = 20;
        }
        /**
         * @return {?}
         */
        ResizeHandlersComponent.prototype.ngAfterViewInit = function () {
            this.setHandlePositions();
        };
        /**
         * @return {?}
         */
        ResizeHandlersComponent.prototype.ngOnChanges = function () {
            if (this.selected) {
                this.setHandlePositions();
            }
        };
        /**
         * @return {?}
         */
        ResizeHandlersComponent.prototype.setHandlePositions = function () {
            var _this = this;
            this.handles.forEach(( /**
             * @param {?} element
             * @param {?} index
             * @return {?}
             */function (element, index) {
                switch (index) {
                    case 0:
                        _this.setHandleStyles(element, { left: '-7px', right: '', top: '-7px', bottom: '' });
                        break;
                    case 1:
                        _this.setHandleStyles(element, { left: '', right: '-7px', top: '-7px', bottom: '' });
                        break;
                    case 2:
                        _this.setHandleStyles(element, { left: '-7px', right: '', top: '', bottom: '-7px' });
                        break;
                    default:
                        _this.setHandleStyles(element, { left: '', right: '-7px', top: '', bottom: '-7px' });
                }
            }));
        };
        /**
         * @param {?} element
         * @param {?} styles
         * @return {?}
         */
        ResizeHandlersComponent.prototype.setHandleStyles = function (element, styles) {
            element.nativeElement.style.left = styles.left;
            element.nativeElement.style.right = styles.right;
            element.nativeElement.style.top = styles.top;
            element.nativeElement.style.bottom = styles.bottom;
        };
        /**
         * @param {?} event
         * @return {?}
         */
        ResizeHandlersComponent.prototype.onPointerDown = function (event) {
            event.stopPropagation();
        };
        /**
         * @param {?} className
         * @param {?} coordinates
         * @return {?}
         */
        ResizeHandlersComponent.prototype.resize = function (className, coordinates) {
            /** @type {?} */
            var parentElement = this.parentElement.nativeElement;
            /** @type {?} */
            var size = this.calculatingPosition(parentElement, className, coordinates);
            if (size.width > this.minimumSize && size.height > this.minimumSize) {
                if (className.includes('TOP-LEFT')) {
                    parentElement.style.left = size.position.x + 'px';
                    parentElement.style.width = size.width + 'px';
                    parentElement.style.top = size.position.y + 'px';
                    parentElement.style.height = size.height + 'px';
                }
                else if (className.includes('TOP-RIGHT')) {
                    parentElement.style.width = size.width + 'px';
                    parentElement.style.top = size.position.y + 'px';
                    parentElement.style.height = size.height + 'px';
                }
                else if (className.includes('BOTTOM-LEFT')) {
                    parentElement.style.left = size.position.x + 'px';
                    parentElement.style.width = size.width + 'px';
                    parentElement.style.height = size.height + 'px';
                }
                else {
                    parentElement.style.width = size.width + 'px';
                    parentElement.style.height = size.height + 'px';
                }
            }
            this.setHandlePositions();
        };
        /**
         * @param {?} parentElement
         * @param {?} className
         * @param {?} coordinates
         * @return {?}
         */
        ResizeHandlersComponent.prototype.calculatingPosition = function (parentElement, className, coordinates) {
            switch (this.rotate) {
                case 90:
                    return {
                        width: className.includes('LEFT') ? parentElement.offsetWidth - coordinates.y : parentElement.offsetWidth + coordinates.y,
                        height: className.includes('TOP') ? parentElement.offsetHeight + coordinates.x : parentElement.offsetHeight - coordinates.x,
                        position: {
                            x: parentElement.offsetLeft + coordinates.y,
                            y: parentElement.offsetTop - coordinates.x
                        }
                    };
                    break;
                case 180:
                    return {
                        width: className.includes('LEFT') ? parentElement.offsetWidth + coordinates.x : parentElement.offsetWidth - coordinates.x,
                        height: className.includes('TOP') ? parentElement.offsetHeight + coordinates.y : parentElement.offsetHeight - coordinates.y,
                        position: {
                            x: parentElement.offsetLeft - coordinates.x,
                            y: parentElement.offsetTop - coordinates.y
                        }
                    };
                    break;
                case 270:
                    return {
                        width: className.includes('LEFT') ? parentElement.offsetWidth + coordinates.y : parentElement.offsetWidth - coordinates.y,
                        height: className.includes('TOP') ? parentElement.offsetHeight - coordinates.x : parentElement.offsetHeight + coordinates.x,
                        position: {
                            x: parentElement.offsetLeft - coordinates.y,
                            y: parentElement.offsetTop + coordinates.x
                        }
                    };
                    break;
                default:
                    return {
                        width: className.includes('LEFT') ? parentElement.offsetWidth - coordinates.x : parentElement.offsetWidth + coordinates.x,
                        height: className.includes('TOP') ? parentElement.offsetHeight - coordinates.y : parentElement.offsetHeight + coordinates.y,
                        position: {
                            x: parentElement.offsetLeft + coordinates.x,
                            y: parentElement.offsetTop + coordinates.y
                        }
                    };
            }
        };
        return ResizeHandlersComponent;
    }());
    ResizeHandlersComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'lib-resize-handlers',
                    template: "<div *ngIf=\"selected\" class=\"resize-handlers\">\n  <div #handle #topLeft\n       class=\"TOP-LEFT handle\"\n       draggable\n       (pointerdown)=\"onPointerDown($event)\"\n       (coordinates)=\"resize(topLeft.className, $event)\"></div>\n  <div #handle #topRight\n       class=\"TOP-RIGHT handle\"\n       draggable\n       (pointerdown)=\"onPointerDown($event)\"\n       (coordinates)=\"resize(topRight.className, $event)\"></div>\n  <div #handle #bottomLeft\n       class=\"BOTTOM-LEFT handle\"\n       draggable\n       (pointerdown)=\"onPointerDown($event)\"\n       (coordinates)=\"resize(bottomLeft.className, $event)\"></div>\n  <div #handle #bottomRight\n       class=\"BOTTOM-RIGHT handle\"\n       draggable\n       (pointerdown)=\"onPointerDown($event)\"\n       (coordinates)=\"resize(bottomRight.className, $event)\"></div>\n</div>\n",
                    styles: [".resize-handlers{border:2px solid #424242;box-sizing:border-box;height:100%;left:0;position:absolute;top:0;width:100%}.handle{background:#fff;height:20px;opacity:1%;position:absolute;width:20px}.TOP-LEFT{border-left:3px solid #424242;border-top:3px solid #424242;left:-7px;top:-7px}.TOP-LEFT:active,.TOP-LEFT:hover{background:transparent;border-bottom:#000;border-radius:unset;border-right:#000}.TOP-RIGHT{border-right:3px solid #424242;border-top:3px solid #424242;right:-7px;top:-7px}.TOP-RIGHT:active,.TOP-RIGHT:hover{background:transparent;border-bottom:#000;border-left:#000;border-radius:unset}.BOTTOM-LEFT{border-bottom:3px solid #424242;border-left:3px solid #424242;bottom:-7px;left:-7px}.BOTTOM-LEFT:active,.BOTTOM-LEFT:hover{background:transparent;border-radius:unset;border-right:#000;border-top:#000}.BOTTOM-RIGHT{border-bottom:3px solid #424242;border-right:3px solid #424242;bottom:-7px;right:-7px}.BOTTOM-RIGHT:active,.BOTTOM-RIGHT:hover{background:transparent;border-left:#000;border-radius:unset;border-top:#000}"]
                }] }
    ];
    /** @nocollapse */
    ResizeHandlersComponent.ctorParameters = function () { return []; };
    ResizeHandlersComponent.propDecorators = {
        parentElement: [{ type: core.Input }],
        rotate: [{ type: core.Input }],
        selected: [{ type: core.Input }],
        handles: [{ type: core.ViewChildren, args: ['handle',] }]
    };
    if (false) {
        /** @type {?} */
        ResizeHandlersComponent.prototype.parentElement;
        /** @type {?} */
        ResizeHandlersComponent.prototype.rotate;
        /** @type {?} */
        ResizeHandlersComponent.prototype.selected;
        /** @type {?} */
        ResizeHandlersComponent.prototype.handles;
        /** @type {?} */
        ResizeHandlersComponent.prototype.minimumSize;
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    var ResizableElementDirective = /** @class */ (function () {
        /**
         * @param {?} el
         * @param {?} container
         * @param {?} componentFactoryResolver
         */
        function ResizableElementDirective(el, container, componentFactoryResolver) {
            this.el = el;
            this.container = container;
            this.componentFactoryResolver = componentFactoryResolver;
            this.rotate = 0;
            this.selected = false;
            this.stopped = new core.EventEmitter();
        }
        /**
         * @return {?}
         */
        ResizableElementDirective.prototype.ngOnInit = function () {
            this.addHandlersToElement();
        };
        /**
         * @return {?}
         */
        ResizableElementDirective.prototype.ngOnChanges = function () {
            if (this.resizeHandlersComponent) {
                this.resizeHandlersComponent.instance.selected = this.selected;
                this.resizeHandlersComponent.instance.rotate = this.rotate;
            }
        };
        /**
         * @return {?}
         */
        ResizableElementDirective.prototype.addHandlersToElement = function () {
            /** @type {?} */
            var component = this.componentFactoryResolver.resolveComponentFactory(ResizeHandlersComponent);
            this.resizeHandlersComponent = this.container.createComponent(component);
            this.resizeHandlersComponent.instance.parentElement = this.el;
            this.resizeHandlersComponent.instance.rotate = this.rotate;
            this.resizeHandlersComponent.instance.selected = this.selected;
            this.el.nativeElement.appendChild(this.resizeHandlersComponent.location.nativeElement);
        };
        /**
         * @return {?}
         */
        ResizableElementDirective.prototype.onPointerDown = function () {
            this.resizeHandlersComponent.instance.selected = true;
        };
        /**
         * @return {?}
         */
        ResizableElementDirective.prototype.onPointerUp = function () {
            this.stopped.emit();
        };
        /**
         * @param {?} targetEvent
         * @return {?}
         */
        ResizableElementDirective.prototype.onWindowPointerUp = function (targetEvent) {
            if (!this.el.nativeElement.contains(targetEvent)) {
                this.resizeHandlersComponent.instance.selected = false;
            }
        };
        return ResizableElementDirective;
    }());
    ResizableElementDirective.decorators = [
        { type: core.Directive, args: [{
                    selector: '[resizable]'
                },] }
    ];
    /** @nocollapse */
    ResizableElementDirective.ctorParameters = function () { return [
        { type: core.ElementRef },
        { type: core.ViewContainerRef },
        { type: core.ComponentFactoryResolver }
    ]; };
    ResizableElementDirective.propDecorators = {
        rotate: [{ type: core.Input }],
        selected: [{ type: core.Input }],
        stopped: [{ type: core.Output }],
        onPointerDown: [{ type: core.HostListener, args: ['pointerdown',] }],
        onPointerUp: [{ type: core.HostListener, args: ['pointerup',] }],
        onWindowPointerUp: [{ type: core.HostListener, args: ['window:pointerup', ['$event.target'],] }]
    };
    if (false) {
        /** @type {?} */
        ResizableElementDirective.prototype.rotate;
        /** @type {?} */
        ResizableElementDirective.prototype.selected;
        /** @type {?} */
        ResizableElementDirective.prototype.stopped;
        /** @type {?} */
        ResizableElementDirective.prototype.resizeHandlersComponent;
        /**
         * @type {?}
         * @private
         */
        ResizableElementDirective.prototype.el;
        /**
         * @type {?}
         * @private
         */
        ResizableElementDirective.prototype.container;
        /**
         * @type {?}
         * @private
         */
        ResizableElementDirective.prototype.componentFactoryResolver;
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    var MutableDivModule = /** @class */ (function () {
        function MutableDivModule() {
        }
        return MutableDivModule;
    }());
    MutableDivModule.decorators = [
        { type: core.NgModule, args: [{
                    declarations: [
                        DraggableElementDirective,
                        ResizableElementDirective,
                        ResizeHandlersComponent
                    ],
                    imports: [
                        common.CommonModule
                    ],
                    entryComponents: [ResizeHandlersComponent],
                    exports: [
                        DraggableElementDirective,
                        ResizableElementDirective,
                    ]
                },] }
    ];

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */

    exports.MutableDivModule = MutableDivModule;
    exports.ɵa = DraggableElementDirective;
    exports.ɵb = ResizableElementDirective;
    exports.ɵc = ResizeHandlersComponent;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=mutable-div.umd.js.map
