import { EventEmitter, Directive, ElementRef, Input, Output, HostListener, Component, ViewChildren, ViewContainerRef, ComponentFactoryResolver, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DraggableElementDirective {
    /**
     * @param {?} el
     */
    constructor(el) {
        this.el = el;
        this.pointerDown = false;
        this.rotate = 0;
        this.coordinates = new EventEmitter();
        this.stopped = new EventEmitter();
        el.nativeElement.style.position = 'absolute';
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onPointerDown(event) {
        event.preventDefault();
        this.pointerDown = true;
        this.setOriginalPosition(event);
    }
    /**
     * @return {?}
     */
    onPointerUp() {
        this.stopped.emit();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onPointerMove(event) {
        if (this.pointerDown) {
            /** @type {?} */
            const element = this.el.nativeElement;
            /** @type {?} */
            const newPosition = this.setNewPosition(event);
            this.setOriginalPosition(event);
            element.style.left = (element.offsetLeft + newPosition.x) + 'px';
            element.style.top = (element.offsetTop + newPosition.y) + 'px';
            this.coordinates.emit({ x: newPosition.x, y: newPosition.y });
        }
    }
    /**
     * @return {?}
     */
    onWindowPointerUp() {
        this.pointerDown = false;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onPanStart(event) {
        this.onPointerDown(event.srcEvent);
    }
    /**
     * @return {?}
     */
    onPanEnd() {
        this.onPointerUp();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onPanMove(event) {
        this.onPointerMove(event.srcEvent);
    }
    /**
     * @return {?}
     */
    onWindowPanEnd() {
        this.onWindowPointerUp();
    }
    /**
     * @private
     * @param {?} event
     * @return {?}
     */
    setOriginalPosition(event) {
        switch (this.rotate) {
            case 90:
                this.originalPosition = {
                    x: event.clientY,
                    y: screen.width - event.clientX
                };
                break;
            case 180:
                this.originalPosition = {
                    x: screen.width - event.clientX,
                    y: screen.height - event.clientY
                };
                break;
            case 270:
                this.originalPosition = {
                    x: screen.height - event.clientY,
                    y: event.clientX
                };
                break;
            default:
                this.originalPosition = {
                    x: event.clientX,
                    y: event.clientY
                };
        }
    }
    /**
     * @private
     * @param {?} event
     * @return {?}
     */
    setNewPosition(event) {
        /** @type {?} */
        let newPosition = {
            x: 0,
            y: 0
        };
        switch (this.rotate) {
            case 90:
                newPosition = {
                    x: event.clientY - this.originalPosition.x,
                    y: (screen.width - event.clientX) - this.originalPosition.y
                };
                break;
            case 180:
                newPosition = {
                    x: (screen.width - event.clientX) - this.originalPosition.x,
                    y: (screen.height - event.clientY) - this.originalPosition.y
                };
                break;
            case 270:
                newPosition = {
                    x: (screen.height - event.clientY) - this.originalPosition.x,
                    y: event.clientX - this.originalPosition.y
                };
                break;
            default:
                newPosition = {
                    x: event.clientX - this.originalPosition.x,
                    y: event.clientY - this.originalPosition.y
                };
        }
        return newPosition;
    }
}
DraggableElementDirective.decorators = [
    { type: Directive, args: [{
                selector: '[draggable]'
            },] }
];
/** @nocollapse */
DraggableElementDirective.ctorParameters = () => [
    { type: ElementRef }
];
DraggableElementDirective.propDecorators = {
    rotate: [{ type: Input }],
    coordinates: [{ type: Output }],
    stopped: [{ type: Output }],
    onPointerDown: [{ type: HostListener, args: ['pointerdown', ['$event'],] }],
    onPointerUp: [{ type: HostListener, args: ['pointerup',] }],
    onPointerMove: [{ type: HostListener, args: ['window:pointermove', ['$event'],] }],
    onWindowPointerUp: [{ type: HostListener, args: ['window:pointerup',] }],
    onPanStart: [{ type: HostListener, args: ['panstart', ['$event'],] }],
    onPanEnd: [{ type: HostListener, args: ['panend',] }],
    onPanMove: [{ type: HostListener, args: ['window:panmove', ['$event'],] }],
    onWindowPanEnd: [{ type: HostListener, args: ['window:panend',] }]
};
if (false) {
    /** @type {?} */
    DraggableElementDirective.prototype.originalPosition;
    /**
     * @type {?}
     * @private
     */
    DraggableElementDirective.prototype.pointerDown;
    /** @type {?} */
    DraggableElementDirective.prototype.rotate;
    /** @type {?} */
    DraggableElementDirective.prototype.coordinates;
    /** @type {?} */
    DraggableElementDirective.prototype.stopped;
    /**
     * @type {?}
     * @private
     */
    DraggableElementDirective.prototype.el;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ResizeHandlersComponent {
    constructor() {
        this.selected = false;
        this.minimumSize = 20;
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.setHandlePositions();
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
        if (this.selected) {
            this.setHandlePositions();
        }
    }
    /**
     * @return {?}
     */
    setHandlePositions() {
        this.handles.forEach((/**
         * @param {?} element
         * @param {?} index
         * @return {?}
         */
        (element, index) => {
            switch (index) {
                case 0:
                    this.setHandleStyles(element, { left: '-7px', right: '', top: '-7px', bottom: '' });
                    break;
                case 1:
                    this.setHandleStyles(element, { left: '', right: '-7px', top: '-7px', bottom: '' });
                    break;
                case 2:
                    this.setHandleStyles(element, { left: '-7px', right: '', top: '', bottom: '-7px' });
                    break;
                default:
                    this.setHandleStyles(element, { left: '', right: '-7px', top: '', bottom: '-7px' });
            }
        }));
    }
    /**
     * @param {?} element
     * @param {?} styles
     * @return {?}
     */
    setHandleStyles(element, styles) {
        element.nativeElement.style.left = styles.left;
        element.nativeElement.style.right = styles.right;
        element.nativeElement.style.top = styles.top;
        element.nativeElement.style.bottom = styles.bottom;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onPointerDown(event) {
        event.stopPropagation();
    }
    /**
     * @param {?} className
     * @param {?} coordinates
     * @return {?}
     */
    resize(className, coordinates) {
        /** @type {?} */
        const parentElement = this.parentElement.nativeElement;
        /** @type {?} */
        const size = this.calculatingPosition(parentElement, className, coordinates);
        if (size.width > this.minimumSize && size.height > this.minimumSize) {
            if (className.includes('TOP-LEFT')) {
                parentElement.style.left = size.position.x + 'px';
                parentElement.style.width = size.width + 'px';
                parentElement.style.top = size.position.y + 'px';
                parentElement.style.height = size.height + 'px';
            }
            else if (className.includes('TOP-RIGHT')) {
                parentElement.style.width = size.width + 'px';
                parentElement.style.top = size.position.y + 'px';
                parentElement.style.height = size.height + 'px';
            }
            else if (className.includes('BOTTOM-LEFT')) {
                parentElement.style.left = size.position.x + 'px';
                parentElement.style.width = size.width + 'px';
                parentElement.style.height = size.height + 'px';
            }
            else {
                parentElement.style.width = size.width + 'px';
                parentElement.style.height = size.height + 'px';
            }
        }
        this.setHandlePositions();
    }
    /**
     * @param {?} parentElement
     * @param {?} className
     * @param {?} coordinates
     * @return {?}
     */
    calculatingPosition(parentElement, className, coordinates) {
        switch (this.rotate) {
            case 90:
                return {
                    width: className.includes('LEFT') ? parentElement.offsetWidth - coordinates.y : parentElement.offsetWidth + coordinates.y,
                    height: className.includes('TOP') ? parentElement.offsetHeight + coordinates.x : parentElement.offsetHeight - coordinates.x,
                    position: {
                        x: parentElement.offsetLeft + coordinates.y,
                        y: parentElement.offsetTop - coordinates.x
                    }
                };
                break;
            case 180:
                return {
                    width: className.includes('LEFT') ? parentElement.offsetWidth + coordinates.x : parentElement.offsetWidth - coordinates.x,
                    height: className.includes('TOP') ? parentElement.offsetHeight + coordinates.y : parentElement.offsetHeight - coordinates.y,
                    position: {
                        x: parentElement.offsetLeft - coordinates.x,
                        y: parentElement.offsetTop - coordinates.y
                    }
                };
                break;
            case 270:
                return {
                    width: className.includes('LEFT') ? parentElement.offsetWidth + coordinates.y : parentElement.offsetWidth - coordinates.y,
                    height: className.includes('TOP') ? parentElement.offsetHeight - coordinates.x : parentElement.offsetHeight + coordinates.x,
                    position: {
                        x: parentElement.offsetLeft - coordinates.y,
                        y: parentElement.offsetTop + coordinates.x
                    }
                };
                break;
            default:
                return {
                    width: className.includes('LEFT') ? parentElement.offsetWidth - coordinates.x : parentElement.offsetWidth + coordinates.x,
                    height: className.includes('TOP') ? parentElement.offsetHeight - coordinates.y : parentElement.offsetHeight + coordinates.y,
                    position: {
                        x: parentElement.offsetLeft + coordinates.x,
                        y: parentElement.offsetTop + coordinates.y
                    }
                };
        }
    }
}
ResizeHandlersComponent.decorators = [
    { type: Component, args: [{
                selector: 'lib-resize-handlers',
                template: "<div *ngIf=\"selected\" class=\"resize-handlers\">\n  <div #handle #topLeft\n       class=\"TOP-LEFT handle\"\n       draggable\n       (pointerdown)=\"onPointerDown($event)\"\n       (coordinates)=\"resize(topLeft.className, $event)\"></div>\n  <div #handle #topRight\n       class=\"TOP-RIGHT handle\"\n       draggable\n       (pointerdown)=\"onPointerDown($event)\"\n       (coordinates)=\"resize(topRight.className, $event)\"></div>\n  <div #handle #bottomLeft\n       class=\"BOTTOM-LEFT handle\"\n       draggable\n       (pointerdown)=\"onPointerDown($event)\"\n       (coordinates)=\"resize(bottomLeft.className, $event)\"></div>\n  <div #handle #bottomRight\n       class=\"BOTTOM-RIGHT handle\"\n       draggable\n       (pointerdown)=\"onPointerDown($event)\"\n       (coordinates)=\"resize(bottomRight.className, $event)\"></div>\n</div>\n",
                styles: [".resize-handlers{border:2px solid #424242;box-sizing:border-box;height:100%;left:0;position:absolute;top:0;width:100%}.handle{background:#fff;height:20px;opacity:1%;position:absolute;width:20px}.TOP-LEFT{border-left:3px solid #424242;border-top:3px solid #424242;left:-7px;top:-7px}.TOP-LEFT:active,.TOP-LEFT:hover{background:transparent;border-bottom:#000;border-radius:unset;border-right:#000}.TOP-RIGHT{border-right:3px solid #424242;border-top:3px solid #424242;right:-7px;top:-7px}.TOP-RIGHT:active,.TOP-RIGHT:hover{background:transparent;border-bottom:#000;border-left:#000;border-radius:unset}.BOTTOM-LEFT{border-bottom:3px solid #424242;border-left:3px solid #424242;bottom:-7px;left:-7px}.BOTTOM-LEFT:active,.BOTTOM-LEFT:hover{background:transparent;border-radius:unset;border-right:#000;border-top:#000}.BOTTOM-RIGHT{border-bottom:3px solid #424242;border-right:3px solid #424242;bottom:-7px;right:-7px}.BOTTOM-RIGHT:active,.BOTTOM-RIGHT:hover{background:transparent;border-left:#000;border-radius:unset;border-top:#000}"]
            }] }
];
/** @nocollapse */
ResizeHandlersComponent.ctorParameters = () => [];
ResizeHandlersComponent.propDecorators = {
    parentElement: [{ type: Input }],
    rotate: [{ type: Input }],
    selected: [{ type: Input }],
    handles: [{ type: ViewChildren, args: ['handle',] }]
};
if (false) {
    /** @type {?} */
    ResizeHandlersComponent.prototype.parentElement;
    /** @type {?} */
    ResizeHandlersComponent.prototype.rotate;
    /** @type {?} */
    ResizeHandlersComponent.prototype.selected;
    /** @type {?} */
    ResizeHandlersComponent.prototype.handles;
    /** @type {?} */
    ResizeHandlersComponent.prototype.minimumSize;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ResizableElementDirective {
    /**
     * @param {?} el
     * @param {?} container
     * @param {?} componentFactoryResolver
     */
    constructor(el, container, componentFactoryResolver) {
        this.el = el;
        this.container = container;
        this.componentFactoryResolver = componentFactoryResolver;
        this.rotate = 0;
        this.selected = false;
        this.stopped = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.addHandlersToElement();
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
        if (this.resizeHandlersComponent) {
            this.resizeHandlersComponent.instance.selected = this.selected;
            this.resizeHandlersComponent.instance.rotate = this.rotate;
        }
    }
    /**
     * @return {?}
     */
    addHandlersToElement() {
        /** @type {?} */
        const component = this.componentFactoryResolver.resolveComponentFactory(ResizeHandlersComponent);
        this.resizeHandlersComponent = this.container.createComponent(component);
        this.resizeHandlersComponent.instance.parentElement = this.el;
        this.resizeHandlersComponent.instance.rotate = this.rotate;
        this.resizeHandlersComponent.instance.selected = this.selected;
        this.el.nativeElement.appendChild(this.resizeHandlersComponent.location.nativeElement);
    }
    /**
     * @return {?}
     */
    onPointerDown() {
        this.resizeHandlersComponent.instance.selected = true;
    }
    /**
     * @return {?}
     */
    onPointerUp() {
        this.stopped.emit();
    }
    /**
     * @param {?} targetEvent
     * @return {?}
     */
    onWindowPointerUp(targetEvent) {
        if (!this.el.nativeElement.contains(targetEvent)) {
            this.resizeHandlersComponent.instance.selected = false;
        }
    }
}
ResizableElementDirective.decorators = [
    { type: Directive, args: [{
                selector: '[resizable]'
            },] }
];
/** @nocollapse */
ResizableElementDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: ViewContainerRef },
    { type: ComponentFactoryResolver }
];
ResizableElementDirective.propDecorators = {
    rotate: [{ type: Input }],
    selected: [{ type: Input }],
    stopped: [{ type: Output }],
    onPointerDown: [{ type: HostListener, args: ['pointerdown',] }],
    onPointerUp: [{ type: HostListener, args: ['pointerup',] }],
    onWindowPointerUp: [{ type: HostListener, args: ['window:pointerup', ['$event.target'],] }]
};
if (false) {
    /** @type {?} */
    ResizableElementDirective.prototype.rotate;
    /** @type {?} */
    ResizableElementDirective.prototype.selected;
    /** @type {?} */
    ResizableElementDirective.prototype.stopped;
    /** @type {?} */
    ResizableElementDirective.prototype.resizeHandlersComponent;
    /**
     * @type {?}
     * @private
     */
    ResizableElementDirective.prototype.el;
    /**
     * @type {?}
     * @private
     */
    ResizableElementDirective.prototype.container;
    /**
     * @type {?}
     * @private
     */
    ResizableElementDirective.prototype.componentFactoryResolver;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class MutableDivModule {
}
MutableDivModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    DraggableElementDirective,
                    ResizableElementDirective,
                    ResizeHandlersComponent
                ],
                imports: [
                    CommonModule
                ],
                entryComponents: [ResizeHandlersComponent],
                exports: [
                    DraggableElementDirective,
                    ResizableElementDirective,
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { MutableDivModule, DraggableElementDirective as ɵa, ResizableElementDirective as ɵb, ResizeHandlersComponent as ɵc };
//# sourceMappingURL=mutable-div.js.map
